/*
  The purpose of this function is to take an image file from the internet and place it into a S3 bucket in AWS using Lambda and Node.js.
  NOTE: If you are looking to resize an image after it has been placed in S3, see http://docs.aws.amazon.com/lambda/latest/dg/with-s3-example-deployment-pkg.html
  About "putObject": The default behavior is, if the file does not already exist at the target S3 location, is to create the file.  If, however, a file
  by the same name and extension exist at the target S3 location, that file will be replaced with what you are putting.
*/

/*
IAM POLICY NOTES FOR S3 (Avoiding "AccessDenied: Access Denied")
The Lambda function usually uses a role that you point to in order for the Lambda function to execute.  If you do not already have a policy in
place to handle interacting with objects in specific locations on S3, you should do so.  Below is an example policy you can create in the IAM.
Once created you can attach the policy to the role you have attached to the Lamda function.  In this example, the ability to get and put objects
is allowed for the bucket "yourbucket" and for a SPECIFIC folder "images".  This means that the role by using the function will NOT be able to 
get or put images in any other bucket or folder, other than "yourbucket/images/":
{
	"Version": "2012-10-17",
	"Statement": [
		{
		"Effect": "Allow",
		"Action": [
			"s3:GetObject",
			"s3:PutObject"
		],
		"Resource": [
			"arn:aws:s3:::yourbucket/images/*"
		]
		}
	]
}
*/

/*
CUSTOM FUNCTIONS TO SIMPLIFY HANDLING RETRIEVAL DATA
*/
function retrievalParseHeaders(data) {
	var result = true;
	var haveValidContent = 0;
	try {
		retServer = data["server"].toString();
		retDate = data["date"].toString();
		retContentType = data["content-type"].toString();
		retContentLength = Number(data["content-length"]);
		retConnection = data["connection"].toString();
		retLastModified = data["last-modified"].toString();
		retAcceptRanges = data["accept-ranges"].toString();
		retVary = data["vary"].toString();
	}
	catch(e) { result = false; }
	/* Validate content type for jpg or png files.  This is not the only way to validate source data, but a good start. */
	if (retContentType.toLowerCase().indexOf("image/jpeg") > -1) { haveValidContent = 1; }
	if (retContentType.toLowerCase().indexOf("image/jpg") > -1) { haveValidContent = 1; }
	if (retContentType.toLowerCase().indexOf("image/png") > -1) { haveValidContent = 1; }
	if (haveValidContent === 0) { retFlag = 1; }
	return result;
}

/*
AWS COMPONENTS
*/
var AWS = require("aws-sdk");
var http = require("http");
var s3 = new AWS.S3();

/*
FILE RETRIEVAL HANDLING VARIABLES
*/
var retBody = [], retBucketParams = {};
var retFlag = 0;
var retServer = "";
var retDate = "";
var retContentType = "";
var retContentLength = 0;
var retConnection = "";
var retLastModified = "";
var retAcceptRanges = "";
var retVary = "";

/*
MAIN FUNCTION
*/
exports.handler = function(event, context) {
	/* Configuration */
	var sourceURL = "http://www.yoursite.com/yourimage.png";
	var storageS3Bucket = "yourbucket/images";
	var storageS3FileName = "yourimage.png";

	http.get(sourceURL, function(result) {
		/* Error */
		result.on('error', function(e) {
			context.done(
				null,
				{
				"message": e.message
				}
			);
		});

		/* Collect Response Stream Data */
		result.on('data', function(chunk) { retBody.push(chunk); });

		/* Req / Res End. */
		result.on('end', function() {
			/* Parse Response Header Data */
			var tmpStatus = retrievalParseHeaders(result.headers);
			if (tmpStatus) {
				if (retFlag === 0) {
					/* Response data found and of correct file type */
					/* Save the file to the S3 bucket */
					retBucketParams = {
						Bucket: storageS3Bucket,
						Key: storageS3FileName,
						Body: Buffer.concat(retBody)
					};
					s3.putObject(retBucketParams, function(err, data) {
						if (err) {
							context.done(
								null,
								{
								"message": "Error with S3: " + err.stack
								}
							);
						}
						else {
							context.done(
								null,
								{
								"message": "Saved to S3.  It is time for waffles."
								}
							);
						}
					});
				}
				else {
					/* Response data but wrong type */
					context.done(
						null,
						{
						"message": "Response data was found but of the wrong type - " + retContentType
						}
					);
				}
			}
			else {
				/* No response data */
				context.done(
					null,
					{
					"message": "No response data was found.  The URL may be incorrect or your request to the image is being blocked."
					}
				);
			}
		});
	});

};